/*
	calc.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "public.h"

const double mypi=3.141592654; //Not every OS defines Pi
long cosc[360]; //Cached cosines
long sinc[360]; //Cached sines. Both not actually correct, warped for my preferred co-ordinate system (Up is 0 degrees, go round clockwise, down is Y positive)
short angc[100][100]; //Angle cache for vector to polar conversion

void initcalc()
{
	double rad; //Angle in radians
	
	for(short i=0;i<360;i++)
	{
		rad=((double)(90-i)/180)*mypi;	
		cosc[i]=(long)(cos(rad)*100);
		sinc[i]=-(long)(sin(rad)*100);
	}
	for(short x=0;x<100;x++)
	{
		for(short y=0;y<100;y++)
		{
			rad=90+(atan((double)y/(double)x)/(mypi/180));
			angc[x][y]=(short)rad;
		}
	}
	fflush(NULL);
}

vct poltovct(pol in,bool hres)
{
	vct out; //Calculated vector
	
	out.xx=(cosc[in.ang]*in.rad);
	out.yy=(sinc[in.ang]*in.rad);
	if(!hres)
	{
		out.xx/=100;
		out.yy/=100;
	}
	return out;
}

pol vcttopol(vct in)
{
	pol out; //Calculated polar
	vct ain; //Abs version of in
	
	if(in.xx<0)
		ain.xx=-in.xx;
	else
		ain.xx=in.xx;
	if(in.yy<0)
		ain.yy=-in.yy;
	else
		ain.yy=in.yy;
	if(ain.xx+ain.yy>32000)
	{
		out.rad=ain.xx+ain.yy;
		if(out.rad==0)
			out.rad==1;
		in.xx/=out.rad/100;
		in.yy/=out.rad/100;
	}
	else
	{
		out.rad=quickroot(ain.xx*ain.xx+ain.yy*ain.yy);
		if(out.rad==0)
			out.rad==1;
		in.xx*=100;
		in.yy*=100;
		in.xx/=out.rad;
		in.yy/=out.rad;
	}
	if(in.xx>=100)
		in.xx=99;
	if(in.xx<=-100)
		in.xx=-99;
	if(in.yy>=100)
		in.yy=99;
	if(in.yy<=-100)
		in.yy=-99;
	if(in.yy>=0)
		if(in.xx>=0)
			out.ang=angc[in.xx][in.yy];
		else
			out.ang=360-angc[-in.xx][in.yy];
	else
		if(in.xx>=0)
			out.ang=180-angc[in.xx][-in.yy];
		else
			out.ang=180+angc[-in.xx][-in.yy];
	if(out.rad==0)
		out.rad=1;
	return out;
}

long quickroot(long x)
{
	long y1,y2; //Iteration variables
	bool ascl; //Scaling used for accuracy?

	if(x<1)
		return 0;
	if(x<10000)
		ascl=true;
	else
		ascl=false;
	if(ascl)
		x*=900;
	y1=x/2;
	for(short i=0;i<15;i++)
	{
		if(y1==0)
			y1=1;
		y2=(y1+x/y1)/2;
		y1=y2;
	}
	if(ascl)
		y2/=30;
	return y2;
}

void seed()
{
	srand(time(NULL));
}

long rnd(long max)
{
	return (rand()/(RAND_MAX/max+1));
}
