/*
	defs.h
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

class frag //A frag object, like phaser fire, torpedo, debris
{
	public:
	
	short self; //Self index in the database
	short typ; //Type: -1 Non-existent  1 Phaser fire  2 Torpedo  3 Debris
	short spr; //Associated sprite
	short trg; //Target
	long x,y; //Position
	vct vel; //Velocity
	short rot; //Rotation frame to use
	short pow; //Power/damage
	long trck; //Tracking power
	short rng; //Range left
	
	void draw(); //Draw the frag
	void physics(); //Move the frag, do collisions
	void home(); //Sub-function of physics, handles homing
	void colldetect(); //Sub-function of physics, handles collision detection
};

class ship //Definition of the spaceship type in game
{
	public:
	
	short self; //Self index in the ship database
	char cls[33]; //Ship class 
	short spr; //Sprite index
	short sspr; //Shield sprite
	short ssnd; //Shield sound
	short fspr; //Frag sprite
	short fsnd; //Frag sound
	short all; //Alliance (0 means dead)
	short stat; //Status: 0 - Disabled  1 - Holding  2 - Active
	long x,y; //Co-ordinates 
	long hrx,hry; //Hi-res X, subdivides the co-ords into -500 - 500. NOT to be used by any function except 'physics', allows smooth movement at low speed
	pol vel; //Velocity
	vct vect; //Caches velocity vector for use by other functions
	long hra; //Hi-res angle
	long hrsp; //Hi-res speed, subdivided -500-500. Only to be used by the 'accel' function
	short trn; //Turning speed
	long mip; //Maximum impulse velocity (<100)
	long aip; //Impulse acceleration
	long mwp; //Maximum warp velocity
	long awp; //Warp acceleration
	long srng; //Sensor range
	short hul; //Hull integrity (related to mass)
	short mss; //Mass
	short pow; //Current power level
	short rpow; //Power regeneration rate
	short mpow; //Maximum power level
	short shd; //Shield level
	short rshd; //Shield regeneration rate
	short mshd; //Maximum shield level
	short shon; //Shields on?
	short clon; //Cloak on?
	short frnd; //Friend target
	short enem; //Enemy target
	short plnt; //Planetary target
	short crew; //Crew size
	struct emp //Equipment emplacement on the ship
	{
		pol pos; //Position (ang=-1 indicates not external)
		short indx; //Equipment item index
		short rdy; //Readiness
		short cap; //Capacity
	}	
	eqp[32]; //Equipment
	
	void draw(); //Draw ship
	void physics(); //Execute physics on the ship, such as motion
	void turn(short dir); //Turns the ship by specified amount`
	void accel(long acc,bool wrp); //Accelerates the ship by specified amount, wrp deciding if you are willing to make the impulse<=>warp transition
	void autonav(); //Executes auto-pilot considerations on ship
	void save(char* path); //Saves the ship data to a file
	void load(void* strm); //Load ship data from a filestream
	void shoot(short wep); //Shoot at target with wep: 1 Phasers  2 Torpedo
	void social(); //Make targetting decisions
	void maintain(); //Does routine work such as recharging shields and power, reloading weapons
	void detonate(void* in); //Detonates the ship as a consequence of incoming frag
	bool see(short indx); //Can the ship see the ship given by index? (sensor range stuff)
	void kill(); //Kill ship and do necessary cleanup
	void colldetect(void* frg); //Checks a given frag for collision
};


class equip //Equipment item type
{
	public:
	
	short self; //Self index in the equipment cache
	char nam[33]; //Name
	short typ; //Type  1:Energy weapon 2:Missile launcher 3:Camo device 4:Hangar
	short mss; //Mass
	short spr; //Associated sprite index
	short pow; //Power consumption
	short rdy; //Readiness cycle time
	short cap; //Capacity
	short rng; //Range
	short trck; //Tracking power
	short acov; //Angle coverage
	
	void load(); //Load equipment data from the database
};

class planet //Planet object
{
	public:
	
	short self; //Self index
	char nam[33]; //Name
	short spr; //Sprite index
	short rot; //Version of sprite to use
	short all; //Alliance (-1 means non-existent)
	long x,y; //Co-ordinates
	short typ; //Planet type 1:Sun 2:Uninhabited 3:Inhabited
	
	void draw(); //Draw the planet
};

class ali //Alliance type
{
	public:
	
	short self; //Self index
	char nam[33]; //Name
	short sdns[8]; //Ship density distribution
	short form; //Formation discipline (during fights)
	short enem[8]; //Alliance index of enemies
	short nsys; //Number of star systems
	short pinh; //Percentage of inhabited planets
		
	void load(); //Load alliance from datafile
};

