/*
	frag.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <stdio.h>
#include "public.h"
#include "defs.h"
#include "layout.h"

extern short me;

const short FRAGCOUNT=512; //Total number of frags in database
frag frags[FRAGCOUNT]; //Frag database

void initfrags()
{
	for(short i=0;i<FRAGCOUNT;i++)
	{
		frags[i].self=i;
		frags[i].typ=-1;
	}
}

void drawfrags()
{
	for(short i=0;i<FRAGCOUNT;i++)
	{
		if(frags[i].typ!=-1)
		{
			frags[i].draw();
		}
	}
}

void dofrags()
{
	for(short i=0;i<FRAGCOUNT;i++)
	{
		if(frags[i].typ!=-1)
		{
			frags[i].physics();
		}
	}	
}

void newfrag(void* in)
{
	for(short i=0;i<FRAGCOUNT;i++)
	{
		if(frags[i].typ==-1)
		{
			frags[i]=*(frag*)in;
			return;
		}
	}
}

void radarfrags(long povx,long povy,long rng)
{
	long ax,ay; //Actual position to draw at
	
	for(short i=0;i<FRAGCOUNT;i++)
	{
		if(frags[i].typ>1)
		{
			ax=frags[i].x-povx;
			ay=frags[i].y-povy;
			if(ax<rng&&ax>-rng&&ay<rng&&ay>-rng)
			{
				ax=(ax*RADAR.w/2)/rng+RADAR.x+RADAR.w/2;
				ay=(ay*RADAR.h/2)/rng+RADAR.y+RADAR.h/2;
				pix(ax,ay,3);
			}
		}
	}
}

void purgefrags(short trg)
{
	for(short i=0;i<FRAGCOUNT;i++)
	{
		if(frags[i].trg==trg)
			frags[i].trg=-1;
	}
}

void frag::draw()
{
	long ax,ay; //Position on screen
	
	getreal(x,y,&ax,&ay);
	if(!(ax>VIEW.x+VIEW.w+50 || ax<VIEW.x-50 || ay>VIEW.y+VIEW.h+50 || ay<VIEW.y-50))
	{
		if(typ==3)
			drawsprite(spr,ax,ay,rot,2);
		else	
			drawsprite(spr,ax,ay,rot,1);
	}
}

void frag::physics()
{
	x+=vel.xx;
	y+=vel.yy;
	rng--;
	if(rng<1)
		typ=-1;
	if(typ==2&&trg!=-1)
	{
		home();
		rot=(rot+5)%36;
	}
	if(typ==1||typ==2)
		colldetect();
}

void frag::home()
{
	vct trv; //Target vector
	pol trp; //Target polar
	ship* strg; //Target
	
	strg=(ship*)getship(trg);
	trv.xx=strg->x-x;
	trv.yy=strg->y-y;
	trp=vcttopol(trv);
	trv.xx=(25*trv.xx*trck)/trp.rad+strg->vect.xx;
	trv.yy=(25*trv.yy*trck)/trp.rad+strg->vect.yy;	
	if(trv.xx>vel.xx)
	{
		vel.xx+=trck;
		if(trv.xx<vel.xx)
			vel.xx=trv.xx;		
	}
	if(trv.xx<vel.xx)
	{
		vel.xx-=trck;
		if(trv.xx>vel.xx)
			vel.xx=trv.xx;
	}
	if(trv.yy>vel.yy)
	{
		vel.yy+=trck;
		if(trv.yy<vel.yy)
			vel.yy=trv.yy;
	}
	if(trv.yy<vel.yy)
	{
		vel.yy-=trck;	
		if(trv.yy>vel.yy)
			vel.yy=trv.yy;		
	}
}

void frag::colldetect()
{		
	ship* strg; //Target
	
	if(trg!=-1)
	{
		strg=(ship*)getship(trg);
		strg->colldetect(this);
	}
}
