/*
	interface.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <SDL.h>
#include <stdlib.h>
#include <string.h>
#include "init.h"
#include "public.h"
#include "defs.h"
#include "layout.h"

short me; //The user

bool flck; //Alternating flicker
bool rtks[512]; //Realtime keys
short lktm; //Last key-press timer
short vtrg; //Viewing target
long povx,povy; //Point of view
long rng; //Sensor range
short mode; //Console mode
short ln; //Console line
long stx[32],sty[32]; //The stars
ship* mine; //User's ship
ship* vshp; //View ship
Uint8 keys[512]; //Keyboard state
bool anav; //Autonavigation?
bool cupd; //Console update?

void drawstars(); //Draw the stars
void drawsensor(); //Draw the sensor display
void drawbars(); //Draw indicator bars
void drawpointer(); //Draw pointer to target
void drawcons(); //Draw the console if it needs updating
void getkeys(); //Get the keyboard state with repeat rate and suchlike accounted for, stash it in 'keys'

void initinterface()
{
	ln=0;
	vtrg=me;
	mode=0;
	for(short i=0;i<512;i++)
		rtks[i]=false;
	rtks[SDLK_LEFT]=true;
	rtks[SDLK_RIGHT]=true;
	rtks[SDLK_UP]=true;
	rtks[SDLK_DOWN]=true;
	rtks[SDLK_SPACE]=true;
	rtks[SDLK_z]=true;
	rtks[SDLK_LSHIFT]=true;
	rtks[SDLK_RSHIFT]=true;
	rtks[SDLK_MINUS]=true;
	rtks[SDLK_EQUALS]=true;
	lktm=0;
	rng=99999999;
	anav=false;
	cupd=true;
}

void control()
{
	SDL_Event evnt; //Event to be polled
	vct ivct;
	pol ipol; //For calculating PoV
	
	mine=(ship*)getship(me);
	SDL_PumpEvents();
	if(!SDL_PollEvent(&evnt))
		evnt.type=0;
	if(evnt.type==SDL_QUIT)
		quit("Successful exit");
	getkeys();
	if(mine->all!=-1)
	{
		if(anav&&mine->plnt!=-1)
		{
			mine->autonav();
		}
		else
		{		
			if(keys[SDLK_LEFT])
				mine->turn(-32767);
			if(keys[SDLK_RIGHT])
				mine->turn(+32767);
			if(keys[SDLK_UP])
			{
				if(keys[SDLK_LSHIFT]||keys[SDLK_RSHIFT])
					mine->accel(+65535,true);
				else
					mine->accel(+65535,false);		
			}

			if(keys[SDLK_DOWN])
			{
				if(keys[SDLK_LSHIFT]||keys[SDLK_RSHIFT])
					mine->accel(-65535,true);
				else
					mine->accel(-65535,false);
			}
		}
		if(keys[SDLK_SPACE])
			mine->shoot(1);
		if(keys[SDLK_z])
			mine->shoot(2);
		if(keys[SDLK_t])
		{
			mine->enem=retargetships((void*)mine,1);
			mine->plnt=-1;
			vtrg=me;
			anav=false;
			cupd=true;
		}
		if(keys[SDLK_e])
		{
			mine->enem=retargetships((void*)mine,2);
			mine->plnt=-1;
			vtrg=me;
			anav=false;
			cupd=true;
		}
		if(keys[SDLK_p])
		{
			mine->plnt=retargetplanets((void*)mine,1);
			mine->enem=-1;
			vtrg=me;
			anav=false;
			cupd=true;
		}
		if(keys[SDLK_s])
		{
			mine->plnt=retargetplanets((void*)mine,2);
			mine->enem=-1;
			vtrg=me;
			anav=false;
			cupd=true;
		}	
		if(keys[SDLK_EQUALS])
			rng-=400;
		if(keys[SDLK_MINUS])
			rng+=400;
		if(keys[SDLK_TAB])
		{
			mode=(mode+1)%3;
			cupd=true;
		}
		if(keys[SDLK_v])
		{
			if(vtrg==mine->enem||mine->enem==-1)
				vtrg=me;
			else
				vtrg=mine->enem;
		}
		if(keys[SDLK_n])
		{
			if(mine->plnt!=-1)
				anav=!anav;
			else
				anav=false;
		}
	}
	if(keys[SDLK_q])
		quit("Successful exit");
	if(rng<1000)
		rng=1000;
	if(rng>mine->srng)
		rng=mine->srng;
	if(mine->enem==-1)
		vtrg=me;
	if(vtrg==me)
		vshp=mine;
	else
		vshp=(ship*)getship(vtrg);
	ipol.rad=1;
	ipol.ang=vshp->vel.ang;
	ivct=poltovct(ipol,true);
	povx=vshp->x+ivct.xx;
	povy=vshp->y+ivct.yy;
}

void drawinterface()
{
	drawsensor();
	drawbars();
	if(cupd)
		drawcons();
	clip(VIEW.x,VIEW.y,VIEW.w,VIEW.h);
	box(VIEW.x,VIEW.y,VIEW.w,VIEW.h,0);
	drawstars();
	drawpointer();
}

void drawstars()
{
	long astx,asty; //Temporary star co-ordinates on screen
	
	for(short i=0;i<32;i++)
	{
		getreal(stx[i],sty[i],&astx,&asty);
		if(astx<VIEW.x || astx>=VIEW.x+VIEW.w || asty<VIEW.y || asty>=VIEW.y+VIEW.h)
		{
			stx[i]=rnd(VIEW.w)+povx-VIEW.w/2;
			sty[i]=rnd(VIEW.h)+povy-VIEW.h/2;
			getreal(stx[i],sty[i],&astx,&asty);
		}
		if(vshp->vel.rad<99)
			pix(astx,asty,6);
		else
			drawsprite(3,astx,asty,((vshp->vel.ang+5)/10)%36,1);
	}
}


void drawsensor()
{
	long ax,ay; //Drawing position
	short locw; //Local area width
	char cord[64]; //Co-ordinate string
	
	flck=!flck;
	clip(RADAR.x,RADAR.y,RADAR.w,RADAR.h);
	box(RADAR.x,RADAR.y,RADAR.w,RADAR.h,0);
	locw=(200*(RADAR.w/2))/rng;
	box(RADAR.x+RADAR.w/2-locw,RADAR.y+RADAR.h/2-locw,2*locw,2*locw,5);
	if(flck)
		pix(RADAR.x+RADAR.w/2,RADAR.y+RADAR.h/2,6);		
	radarships(mine->x,mine->y,rng);
	radarfrags(mine->x,mine->y,rng);
	radarplanets(mine->x,mine->y,rng,mine->plnt);
	sprintf(cord,"%ld , %ld",mine->x/100,mine->y/100);
	string(cord,RADAR.x+1,RADAR.y+1);
}

void drawbars()
{
	long h,y; //Height and y position of bar

	clip(BARS.x,BARS.y,BARS.w,BARS.h);
	box(BARS.x,BARS.y,BARS.w,BARS.h,4);
	y=-(mine->shd*BARS.h)/mine->mshd+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x,y,8,h,1);
	y=-(mine->pow*BARS.h)/mine->mpow+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x+8,y,8,h,2);
	y=-(mine->hul*BARS.h)/mine->mss+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x+16,y,8,h,6);
	if(mine->vel.rad<=mine->mip)
		y=-(mine->vel.rad*BARS.h)/mine->mip+BARS.y+BARS.h;
	else
		y=-(mine->vel.rad*BARS.h)/mine->mwp+BARS.y+BARS.h;
	h=BARS.y+BARS.h-y;
	box(BARS.x+24,y,8,h,3);
}

void drawcons()
{
	cupd=false;
	clip(CONS.x,CONS.y,CONS.w,CONS.h);
	if(mode==0)
	{
		printcons(NULL);
		printcons("Operations manual");
		printcons("");
		printcons("Cursors - Speed and heading");
		printcons("<SHIFT> - Warp/impulse");
		printcons("  <TAB> - Switch console modes");
		printcons("      E - Target hostiles");
		printcons("      T - Target vessel");
		printcons("      P - Target planet");
		printcons("      S - Target star system");
		printcons("<SPACE> - Fire phasers");
		printcons("      Z - Launch torpedo");
		printcons("      N - Toggle autonavigation");
		printcons("    -/+ - Zoom sensor display");
		printcons("      Q - Quit");
	}
	if(mode==1)
	{
		printcons(NULL);
		if(mine->enem==-1&&mine->plnt==-1)
		{
			printcons("No target selected");
			spritecons(5);
		}
		else
		{
			if(mine->enem!=-1)
			{
				scanship((void*)mine,mine->enem);
				cupd=true;
			}
			if(mine->plnt!=-1)
				scanplanet((void*)mine,mine->plnt);
		}
	}
	if(mode==2)
	{
		printcons(NULL);
		printcons("Starchart");
		starchart(mine->x,mine->y,mine->plnt);
		cupd=true;
	}	
}

bool see(long x,long y)
{
	if((x-mine->x)<rng && (x-mine->x)>-rng && (y-mine->y)<rng && (y-mine->y)>-rng)
		return true;
	else
		return false;
}

void drawpointer()
{
	vct vtrg; //Vector to target
	pol ptrg; //Polar to target
	ship* strg; //Ship target
	planet* pltrg; //Planet target
		
	vtrg.xx=0;
	vtrg.yy=0;
	if(mine->enem!=-1)
	{
		strg=(ship*)getship(mine->enem);
		vtrg.xx=strg->x-povx;
		vtrg.yy=strg->y-povy;
	}
	if(mine->plnt!=-1)
	{
		pltrg=(planet*)getplanet(mine->plnt);
		vtrg.xx=pltrg->x-povx;
		vtrg.yy=pltrg->y-povy;
	}
	if(vtrg.xx>180||vtrg.xx<-180||vtrg.yy>180||vtrg.yy<-180)
	{
		ptrg=vcttopol(vtrg);
		ptrg.rad=100;
		vtrg=poltovct(ptrg,false);
		vtrg.xx+=VIEW.x+VIEW.w/2;
		vtrg.yy+=VIEW.y+VIEW.h/2;
		drawsprite(2,vtrg.xx,vtrg.yy,((ptrg.ang+5)/10)%36,1);	
	}
}

void printcons(char* text)
{
	if(text==NULL)
	{
		box(CONS.x,CONS.y,CONS.w,CONS.h,4);
		ln=0;
	}
	else
	{
		string(text,CONS.x+2,CONS.y+ln*6);
		ln++;
	}
	if(ln==26)
		ln=0;
}

void spritecons(short indx)
{
	drawsprite(indx,CONS.x+CONS.w/2,CONS.y+CONS.h/2,0,0);
}

void getreal(long x,long y,long* ax,long* ay)
{
	*ax=x-povx+VIEW.x+VIEW.w/2;
	*ay=y-povy+VIEW.y+VIEW.h/2;
}

void getkeys()
{
	int nkys; //Number of keys
	Uint8* keyb; //Keyboard snapshot
	bool key; //Key press?
	
	key=false;
	keyb=SDL_GetKeyState(&nkys);
	for(short i=0;i<nkys;i++)
	{
		keys[i]=keyb[i];
		if(!rtks[i]&&keys[i])
			key=true;
	}
	if(lktm>0)
	{
		for(short i=0;i<nkys;i++)
		{
			if(!rtks[i])
				keys[i]=false;
		}
	}
	else
	{
		if(key)
			lktm=6;
	}	
	lktm--;
	if(lktm<0)
		lktm=0;
}

