/*
	planet.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <stdio.h>
#include <string.h>
#include "public.h"
#include "defs.h"
#include "layout.h"

extern short me;

const short PLANETCOUNT=256; //Total number of planets to be implemented
char n1[][16]={"Er","E","Cl","St","Vu","Mi","Je","Vi","Pl","J","S","Tw","S","Ed","Br","X"};
char n2[][16]={"di","r","au","os","en","per","pit","le","m","ax","on","ew","aw","sel","st","oo"};
char n3[][16]={"an","e","a","","is","ia","is","anis","ol","oon","","","arb","bre","el","ni"}; //Name building database
planet plns[PLANETCOUNT]; //Planets in game
short slo=100,shi=102,ulo=110,uhi=113,ilo=120,ihi=121; //Boundings for planet sprites

void newplanet(planet* in); //Puts the given planet into existence
void generatename(char* put); //Generates a planet name, inserts into put

void initplanets()
{
	for(short i=0;i<PLANETCOUNT;i++)
		plns[i].all=-1;
}

void newsystem(long x,long y,short all)
{
	planet npln; //New planet scratchpad
	short n; //Number of planets in system
	ali* tali; //System alliance
	char snam[33]; //System name
	
	tali=(ali*)getali(all);
	n=rnd(8);
	if(rnd(10)>5)
		n+=3;
	npln.all=all;
	npln.x=x;
	npln.y=y;
	for(short i=0;i<n;i++)
	{
		npln.rot=rnd(36);
		if(i==0)
		{
			npln.typ=1;
			npln.spr=slo+rnd(shi-slo+1);
			generatename(npln.nam);
			strcpy(snam,npln.nam);
		}
		else
		{
			if(rnd(100)>tali->pinh)
			{
				npln.typ=2;
				npln.spr=ulo+rnd(uhi-ulo+1);
				sprintf(npln.nam,"%s %hd",snam,i);
			}
			else
			{
				npln.typ=3;
				npln.spr=ilo+rnd(ihi-ilo+1);
				generatename(npln.nam);
			}
		}			
		newplanet(&npln);
		npln.x=x+rnd(10000)-5000;
		npln.y=y+rnd(10000)-5000;
	}
}

void generatename(char* put)
{
	short s1,s2,s3; //Syllable selectors

	s1=rnd(16);
	s2=rnd(16);
	s3=rnd(16);
	sprintf(put,"%s%s%s",n1[s1],n2[s2],n3[s3]);
}

void newplanet(planet* in)
{
	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all==-1)
		{
			plns[i]=*in;
			plns[i].self=i;
			return;
		}
	}
	debug("Doh!",in->all);
}

void drawplanets()
{
	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all!=-1)
			plns[i].draw();
	}
}

void* getplanet(short indx)
{
	return &plns[indx];
}

void radarplanets(long povx,long povy,long rng,short trg)
{
	long ax,ay; //Actual position to draw at

	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all!=-1)
		{
			ax=plns[i].x-povx;
			ay=plns[i].y-povy;
			if(ax<rng&&ax>-rng&&ay<rng&&ay>-rng)
			{
				ax=(ax*RADAR.w/2)/rng+RADAR.x+RADAR.w/2;
				ay=(ay*RADAR.h/2)/rng+RADAR.y+RADAR.h/2;
				if(i==trg)
					box(ax-1,ay-1,3,3,3);
				pix(ax,ay,2);
			}
		}
	}
}


void starchart(long povx,long povy,short trg)
{
	long ax,ay; //Actual position to draw at
	ali* tali; //Alliance of targetted planet
	
	for(short i=0;i<PLANETCOUNT;i++)
	{
		if(plns[i].all!=-1&&plns[i].typ==1)
		{
			ax=plns[i].x/(LIMIT/(CONS.w/2))+CONS.x+CONS.w/2;
			ay=plns[i].y/(LIMIT/(CONS.h/2))+CONS.y+CONS.h/2;
			if(i==trg)
			{
				tali=(ali*)getali(plns[i].all);
				box(ax-1,ay-1,3,3,3);
				string(plns[i].nam,ax+2,ay+2);
				string(tali->nam,ax+2,ay+8);
			}
			pix(ax,ay,2);
		}
	}
	ax=povx/(LIMIT/(CONS.w/2))+CONS.x+CONS.w/2;
	ay=povy/(LIMIT/(CONS.h/2))+CONS.y+CONS.h/2;
	box(ax-1,ay-1,3,3,1);
}

short retargetplanets(void* mine,short typ)
{
	short t; //Target select

	t=((ship*)mine)->plnt;
	if(t==-1)
		t=0;
	for(short i=0;i<PLANETCOUNT;i++)
	{
		t=(t+1)%PLANETCOUNT;
		if(plns[t].all!=-1)
		{
			if(see(plns[t].x,plns[t].y)&&typ==1)
				break;
			if(plns[t].typ==1&&typ==2)
				break;
		}
		if(i==PLANETCOUNT-1)
		{
			t=-1;
			break;
		}		
	}
	return t;
}

void scanplanet(void* mine,short indx)
{
	char txt[64]; //Scratchpad	
	ali* tali; //This planet's alliance
	
	tali=(ali*)getali(plns[indx].all);
	printcons(plns[indx].nam);
	sprintf(txt,"Alignment:%s",tali->nam);
	printcons(txt);
	if(plns[indx].typ==1)
		printcons("Star");
	if(plns[indx].typ==2)
		printcons("Uninhabited planet");
	if(plns[indx].typ==3)
		printcons("Inhabited planet");
	spritecons(plns[indx].spr);
}


short shipyard()
{
	short indx; //Index of planet to use
	
	indx=rnd(PLANETCOUNT);
	if(plns[indx].typ==2)
		return newship(getspawn(plns[indx].all),plns[indx].x+rnd(100)-50,plns[indx].y+rnd(100)-50,plns[indx].all);
	return -1;
}

void planet::draw()
{
	long ax,ay; //Position on screen
	
	getreal(x,y,&ax,&ay);
	if(!(ax>VIEW.x+VIEW.w+50 || ax<VIEW.x-50 || ay>VIEW.y+VIEW.h+50 || ay<VIEW.y-50))
		drawsprite(spr,ax,ay,rot,1);
}
