/*
	ship.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <string.h>
#include <stdio.h>
#include <math.h>
#include "config.h"
#include "public.h"
#include "defs.h"
#include "layout.h"

extern short me;

const short SHIPCOUNT=128; //Total number of ships to be implemented in the game
const short SHIPDBCOUNT=16; //Total number of ship types in database
ship sdb[SHIPDBCOUNT]; //Ship type database
ship ships[SHIPCOUNT]; //Main ship list
short frst; //Firing strobe
short j; //Ship behaviour strobe

void initships()
{
	char* path; //Path to load ship data from
	FILE* strm; //Filestream to use
	
	j=0;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		ships[i].all=-1;
		ships[i].self=i;
	}
	memtest(path=new char[strlen(DATADIR)+16]);	
	for(short i=0;i<SHIPDBCOUNT;i++)
	{
		sdb[i].cls[0]=0;
		sprintf(path,"%s%sships%s%hd.shp",DATADIR,PATHSEP,PATHSEP,i);
		strm=fopen(path,"r");
		if(strm!=NULL)
		{
			sdb[i].load((void*)strm);
			fclose(strm);
		}
	}
}

void drawships()
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1)
		{
			ships[i].draw();
		}
	}
}

void doships()
{
	frst=(frst+1)%500;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1||i==me)
		{
			ships[i].physics();
			ships[i].maintain();
		}
	}
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1&&i!=me)
		{
			if(ships[i].stat>1)
				ships[i].autonav();
			if(ships[i].enem!=-1)
			{
				if(ships[i].see(ships[i].enem));
				{
					if((frst+5*i)%10>3)
						ships[i].shoot(1);
					if((frst+5*i)%250<10)
						ships[i].shoot(2);
				}
			}
		}	
	}
	for(short i=0;i<5;i++)
	{
		j=(j+1)%SHIPCOUNT;
		if(j!=me)
			ships[j].social();
	}
}

short newship(short indx,long x,long y,short all)
{
	char err[33]; //Error string
	
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all==-1)
		{
			if(sdb[indx].cls[0]==0)
			{
				sprintf(err,"Ship data index %hd not available",indx);
				quit(err);			
			}
			ships[i]=sdb[indx];
			ships[i].self=i;
			ships[i].x=x;
			ships[i].y=y;
			ships[i].all=all;
			ships[i].vel.ang=rnd(360);
			ships[i].vel.rad=0;
			ships[i].shd=ships[i].mshd;
			ships[i].pow=ships[i].mpow;
			ships[i].frnd=-1;
			ships[i].enem=-1;
			ships[i].plnt=-1;
			ships[i].stat=2;
			ships[i].hul=ships[i].mss;
			ships[i].shon=1;
			ships[i].clon=0;
			return i;
		}
	}
	return -1;
}

void* getship(short indx)
{
	return &ships[indx];
}

void radarships(long povx,long povy,long rng)
{
	long ax,ay; //Actual position to draw at
	
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1&&i!=me)
		{
			ax=ships[i].x-povx;
			ay=ships[i].y-povy;
			if(ax<rng&&ax>-rng&&ay<rng&&ay>-rng)
			{
				ax=(ax*RADAR.w/2)/rng+RADAR.x+RADAR.w/2;
				ay=(ay*RADAR.h/2)/rng+RADAR.y+RADAR.h/2;
				if(i==ships[me].enem)
				{
					box(ax-1,ay-1,3,3,3);
				}
				pix(ax,ay,6);
			}
		}
	}
}

short retargetships(void* mine,short typ)
{
	short t; //Target select

	t=((ship*)mine)->enem;
	if(t==-1)
		t=0;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		t=(t+1)%SHIPCOUNT;
		if(ships[t].all!=-1&&t!=me)
		{
			if(see(ships[t].x,ships[t].y))
			{
				if(typ==1)
					break;
				if(typ==2)
					if(opposes(((ship*)mine)->all,ships[t].all))
						break;
			}					
		}
		if(i==SHIPCOUNT-1)
		{
			t=-1;
			break;
		}		
	}
	return t;
}


void scanship(void* mine,short indx)
{
	char txt[64]; //Scratchpad
	ali* tali; //This ship's alliance

	tali=(ali*)getali(ships[indx].all);
	sprintf(txt,"%s [%hd]",ships[indx].cls,(indx*10001)%2409);
	printcons(txt);
	if(opposes(((ship*)mine)->all,ships[indx].all))
		sprintf(txt,"Alignment:%s(hostile)",tali->nam);
	else
		sprintf(txt,"Alignment:%s",tali->nam);
	printcons(txt);
	sprintf(txt,"Shields:%hd/%hd",ships[indx].shd,ships[indx].mshd);
	printcons(txt);
	spritecons(ships[indx].spr);
}

void ship::draw()
{
	long ax,ay; //Actual position on screen
	short rot; //Rotation frame to use
	short hw,hh; //Sprite dimensions

	getreal(x,y,&ax,&ay);
	if(!(ax>VIEW.x+VIEW.w+50 || ax<VIEW.x-50 || ay>VIEW.y+VIEW.h+50 || ay<VIEW.y-50))
	{
		rot=((vel.ang+5)/10)%36;
		drawsprite(spr,ax,ay,rot,1);
		if(self==ships[me].enem)
		{
			getspritedims(spr,rot,1,&hw,&hh);
			drawsprite(0,ax,ay-hh,0,1);
			drawsprite(0,ax,ay+hh,18,1);
			drawsprite(0,ax-hw,ay,27,1);
			drawsprite(0,ax+hw,ay,9,1);
		}
	}
}

void ship::physics()
{
	vct mov; //Resultant motion
	pol avel; //Adjusted velocity
	
	avel.ang=vel.ang;
	if(vel.rad>mip&&vel.rad<100)
		avel.rad=mip;
	else
		avel.rad=vel.rad;
	mov=poltovct(avel,true);
	hrx+=mov.xx;
	hry+=mov.yy;
	vect.xx=hrx/50;
	vect.yy=hry/50;
	if(hrx<-50)
		hrx=50+hrx%50;
	else
		hrx=hrx%50;
	if(hry<-50)
		hry=50+hry%50;
	else
		hry=hry%50;
	x+=vect.xx;
	y+=vect.yy;
}

void ship::turn(short dir)
{
	short atrn; //Actual turning power
	
//	if(vel.rad>=100)
//		atrn=trn/(vel.rad/200);
//	else
		atrn=trn;
	if(dir>trn)
		dir=atrn;
	if(dir<-trn)
		dir=-atrn;
	hra+=dir;
	vel.ang+=hra/50;
	hra=(50+hra)%50;
	vel.ang=(vel.ang+360)%360;
}

void ship::accel(long acc,bool wrp)
{
	if(vel.rad<=mip)
	{
		if(acc<0)
			hrsp-=aip;
		if(acc>0)
			hrsp+=aip;
		vel.rad+=hrsp/50;
		if(hrsp<-50)
			hrsp=50+hrsp%50;
		else
			hrsp=hrsp%50;
	}
	if(vel.rad>mip&&vel.rad<100)
	{
		vel.rad=mip;
		if(acc>0&&wrp)
			vel.rad=100;
		if(acc<=0&&wrp)
			vel.rad=mip;
	}
	if(vel.rad>=100)
	{
		if(acc>awp)
			acc=awp;
		if(acc<-awp)
			acc=-awp;
		vel.rad+=acc;
		if(vel.rad<100&&!wrp)
			vel.rad=100;
	}
	if(vel.rad<0)
		vel.rad=0;
	if(vel.rad>mwp)
		vel.rad=mwp;
}

void ship::autonav()
{
	pol dpol; //Desired polar to target
	vct vect; //Vector to target
	short dd; //Directional difference
	short tol; //Angular tolerance
	short trg; //Target
	long acc; //Available acceleration
	bool erat; //erattic approach?
	planet* pln; //Targeting planet
	
	trg=-1;
	if(plnt!=-1&&stat!=3)
	{
		trg=plnt;
		erat=false;
		pln=(planet*)getplanet(plnt);
	}
	if(enem!=-1)
	{
		trg=enem;
		erat=true;
		pln=NULL;
	}
	if(frnd!=-1&&(stat==2||enem==-1))
	{
		trg=frnd;
		erat=false;
		pln=NULL;
	}
	if(trg==-1)
		return;
	if(vel.rad<100)
		acc=aip;
	else
	{
		acc=awp*100;
		erat=false;
	}
	if(pln==NULL)
	{
		vect.xx=ships[trg].x;
		vect.yy=ships[trg].y;
	}
	else
	{
		vect.xx=pln->x;
		vect.yy=pln->y;
	}
	vect.xx-=x+(self*27)%(mss+1)-(mss/2);
	vect.yy-=y+(self*103)%(mss+1)-(mss/2);
	dpol=vcttopol(vect);
	tol=1+(trn/50)*(self%3+1);
	dd=(dpol.ang-vel.ang);
	if(dd>180)
		dd=dd-360;
	if(dd<-180)
		dd=dd+360;
	if(dpol.rad<200&&erat)
		tol=360;
	if(dd<tol&&dd>-tol)
	{
		if(vel.rad>=100)
			dpol.rad-=vel.rad*2;
		if(pln==NULL)
			if(erat)
				dpol.rad=quickroot(2*acc)*quickroot(dpol.rad+20)/20+ships[trg].vel.rad;
			else
				dpol.rad=quickroot(2*acc)*quickroot(dpol.rad-mss/2-20)/20+ships[trg].vel.rad;
		else
		{
			dpol.rad=quickroot(2*acc)*quickroot(dpol.rad-mss/2-20)/20;
		}
	}
	else
	{
		dpol.rad=0;
	}
	if(dpol.rad>mip&&dpol.rad<100)
		dpol.rad=mip;
	turn(dd*50);
	accel(dpol.rad-vel.rad,true);
}

void ship::save(char* path)
{
/*	FILE* strm; //Filestream to use
	
	//NOT STABLE WITH NEW FILE FORMAT
	return;
	//NOT STABLE WITH NEW FILE FORMAT
	
	strm=fopen(path,"w+");
	fprintf(strm,"%s\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%ld\n%ld\n",cls,spr,sspr,ssnd,fspr,fsnd,all,stat,x,y);
	fprintf(strm,"%hd\n%ld\n%hd\n%ld\n%ld\n%ld\n%ld\n",vel.ang,vel.rad,trn,mip,aip,mwp,awp);
	fprintf(strm,"%ld\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n",srng,hul,mss,pow,rpow,mpow,shd,rshd,mshd,shon,clon);
	fprintf(strm,"%hd\n%hd\n%hd\n%hd\n",frnd,enem,plnt,crew);
	for(short i=0;i<32;i++)
		fprintf(strm,"\n%hd\n%ld\n%hd\n%hd\n%hd\n",eqp[i].pos.ang,eqp[i].pos.rad,eqp[i].indx,eqp[i].rdy,eqp[i].cap);
	fclose(strm);*/
}

void ship::load(void* strm)
{
	char atsc[33]; //Attribute scratchpad
	
	extractstring(strm,"Class",cls);
	spr=extractlong(strm,"ShipSprite");
	sspr=extractlong(strm,"ShieldSprite");
	ssnd=extractlong(strm,"ShieldSound");
	fspr=extractlong(strm,"FragSprite");
	fsnd=extractlong(strm,"FragSound");
	all=extractlong(strm,"Team");
	stat=extractlong(strm,"Status");
	x=extractlong(strm,"XLoc");
	y=extractlong(strm,"YLoc");
	vel.ang=extractlong(strm,"Heading");
	vel.rad=extractlong(strm,"Speed");
	trn=extractlong(strm,"TurnRate");
	mip=extractlong(strm,"SublightLimit");
	aip=extractlong(strm,"SublightAcceleration");
	mwp=extractlong(strm,"WarpLimit");
	awp=extractlong(strm,"WarpAcceleration");
	srng=extractlong(strm,"SensorRange");
	hul=extractlong(strm,"HullStrength");
	mss=extractlong(strm,"HullMass");
	pow=extractlong(strm,"Power");
	rpow=extractlong(strm,"PowerRecharge");
	mpow=extractlong(strm,"PowerLimit");
	shd=extractlong(strm,"Shield");
	rshd=extractlong(strm,"ShieldRecharge");
	mshd=extractlong(strm,"ShieldLimit");
	shon=extractlong(strm,"ShieldStatus");
	clon=extractlong(strm,"CloakStatus");
	frnd=extractlong(strm,"FriendTarget");
	enem=extractlong(strm,"EnemyTarget");
	plnt=extractlong(strm,"PlanetTarget");
	crew=extractlong(strm,"CrewSize");	
	for(short i=0;i<32;i++)
	{
		sprintf(atsc,"Slot%hdAngle",i);
		eqp[i].pos.ang=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdRadius",i);
		eqp[i].pos.rad=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdItem",i);
		eqp[i].indx=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdReadiness",i);
		eqp[i].rdy=extractlong(strm,atsc);
		sprintf(atsc,"Slot%hdCapacity",i);
		eqp[i].cap=extractlong(strm,atsc);
	}
	hrx=0;
	hry=0;
	hrsp=0;
	hra=0;
}

void ship::shoot(short wep)
{
	equip* foc; //Equipment item in focus
	frag out; //Frag scratchpad
	vct vtmp; //Vector scratchpad
	pol ptmp; //Polar scratchpad
	pol trg; //Target
	short acov; //Angle coverage
	short dif; //Angle difference
	short rng; //Range
	bool cov; //Covered by angle and range?		
	
	if(enem==-1)
		return;
	if(wep==1&&vel.rad>99)
		return;
	vtmp.xx=ships[enem].x+ships[enem].vect.xx-x-vect.xx;
	vtmp.yy=ships[enem].y+ships[enem].vect.yy-y-vect.yy;
	trg=vcttopol(vtmp);	
	for(short i=0;i<32;i++)
	{
		if(eqp[i].indx!=-1)
		{
			foc=(equip*)getequip(eqp[i].indx);
			cov=false;
			if(wep==1)
				rng=foc->rng*foc->trck*5;
			if(wep==2)
				rng=srng;
			if(trg.rad<=rng)
			{
				dif=trg.ang-(vel.ang+eqp[i].pos.ang);
				if(dif>180)
					dif=dif-360;
				if(dif<-180)
					dif=dif+360;
				if(dif<=foc->acov&&dif>=-foc->acov)
					cov=true;		
			}
			if(cov&&eqp[i].rdy==0&&foc->typ==wep&&pow>=foc->pow&&eqp[i].cap>0)
			{
				out.typ=wep;
				out.spr=foc->spr;
				ptmp=eqp[i].pos;
				ptmp.ang=(ptmp.ang+vel.ang)%360;
				vtmp=poltovct(ptmp,false);
				ptmp.ang=trg.ang;
				if(wep==1)
					pow-=foc->pow;
				ptmp.rad=5*foc->trck;
				out.vel=poltovct(ptmp,false);
				out.vel.xx+=vect.xx;
				out.vel.yy+=vect.yy;
				out.x=x+vtmp.xx-out.vel.xx;
				out.y=y+vtmp.yy-out.vel.yy;
				out.rot=((trg.ang+5)/10)%36;
				out.rng=foc->rng;
				out.trck=foc->trck;
				out.trg=enem;
				out.pow=foc->pow;
				newfrag(&out);
				eqp[i].rdy=foc->rdy;
				if(wep==2)
				{
					eqp[i].cap--;
					break;
				}
			}
		}
	}
}

void ship::social()
{
	short t; //Potential target
	vct tvct; //Temporary vector
	pol epol; //Polar to enemy
	pol fpol; //Polar to friend
	ali* tali; //This ship's alliance
	
	if(enem==-1)
	{
		for(short i=0;i<SHIPCOUNT;i++)
		{
			t=rnd(SHIPCOUNT);
			if(opposes(all,ships[t].all))
			{
				if(see(t))
				{
					enem=t;
					return;
				}
			}
		}
	}
	else
	{
		if(frnd!=-1)
		{
			tali=(ali*)getali(all);
			tvct.xx=ships[frnd].x-x;
			tvct.yy=ships[frnd].y-y;
			fpol=vcttopol(tvct);
			tvct.xx=ships[enem].x-x;
			tvct.yy=ships[enem].y-y;
			epol=vcttopol(tvct);
			if(epol.rad/fpol.rad>tali->form&&see(enem))
				stat=3;
			else
				stat=2;
		}
	}
}

void ship::maintain()
{
	for(short i=0;i<32;i++)
	{
		if(eqp[i].rdy>0)
			eqp[i].rdy--;
	}
	if(pow<mpow)
		pow+=rpow;
	if(pow>mpow)
		pow=mpow;	
	if(shon==1&&shd<mshd&&pow>=rshd)
	{
		shd+=rshd;
		pow-=rshd;
		if(shd>mshd)
			shd=mshd;
	}
	if(shon==0)
		shd/=2;
	if(enem!=-1)
	{
		if(ships[enem].all==-1)
			enem=-1;
	}
	if(frnd!=-1)
	{
		if(ships[frnd].all==-1)
			frnd=-1;
	}
}

void ship::detonate(void* in)
{
	frag out; //Scratchpad for making debris
	short spd; //Speed of explosion

	spd=3+(((frag*)in)->pow)/mss;
	out.typ=3;
	for(short i=0;i<4;i++)
	{
		out.pow=rnd(9)-5;
		out.spr=fspr;
		out.rot=rnd(36);
		out.rng=rnd(80)+20;
		if(i%3==0)
		{
			out.x=(3*(((frag*)in)->x-x)/5)+x-vect.xx;
			out.y=(3*(((frag*)in)->y-y)/5)+y-vect.yy;
			out.vel.xx=vect.xx;
			out.vel.yy=vect.yy;
			out.spr=1;
			out.rng=rnd(10)+5;
		}
		out.x+=rnd(8)-4;
		out.y+=rnd(8)-4;
		out.vel.xx+=rnd(spd)-spd/2;
		out.vel.yy+=rnd(spd)-spd/2;
		out.trg=-1;
		newfrag(&out);
	}
}

bool ship::see(short indx)
{
	if(indx==-1)
		return false;
	if((ships[indx].x-x)<srng&&(ships[indx].x-x)>-srng&&(ships[indx].y-y)<srng&&(ships[indx].y-y)>-srng&&ships[indx].all!=-1)
		return true;
	else
		return false;

}

void ship::colldetect(void* in)
{
	short rot; //Target rotation
	long x1,y1,x2,y2,xx,yy; //Target bounding box
	bool hit; //Hit?
	short hw,hh; //Sprite dimensions
	frag* inu; //Casted 'in' to use
	
	inu=(frag*)in;
	if((inu->x)-x<400 && (inu->x)-x>-400 && (inu->y)-y<400 && (inu->y)-y>-400)
	{
		hit=false;
		rot=(vel.ang+5)/10;
		xx=(inu->vel.xx-vect.xx)/2;
		yy=(inu->vel.yy-vect.yy)/2;
		if(xx<0)
			xx=-xx;
		if(yy<0)
			yy=-yy;
		getspritedims(spr,rot,1,&hw,&hh); 
		x1=x-hw-xx;
		y1=y-hh-yy;
		x2=x+hw+xx;
		y2=y+hh+yy;
		if(inu->x>x1 && inu->x<x2 && inu->y>y1 && inu->y<y2)
		{
			shd-=inu->pow;
			if(shd>0)
			{
				inu->vel.xx+=vect.xx;
				inu->vel.yy+=vect.yy;
				inu->rng=1;
				inu->spr=sspr;
				inu->rot=rnd(36);
				inu->typ=3;
			}
			else
			{
				inu->rng=0;
				detonate(inu);
				shd=0;
			}
			hul-=(inu->pow*20)/(shd+1);
			if(hul<0)
			{
				hul=0;
				for(short i=0;i<mss/50+2;i++)
				{	
					detonate(inu);
					inu->x=x1+rnd(x2-x1);
					inu->y=y1+rnd(y2-y1);
				}
				kill();
			}
			inu->trg=-1;
		}
	}
}





void ship::kill()
{
	short hevi,hevm; //Find heaviest to lead the fleet
	
	hevm=-1;
	all=-1;
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1&&ships[i].frnd==self)
		{
			if(ships[i].mss>hevm)
			{
				hevm=ships[i].mss;
				hevi=i;
			}
		}
	}
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1)
		{
			if(ships[i].frnd==self&&i!=hevi)
				ships[i].frnd=hevi;
			if(ships[i].enem==self)
				ships[i].enem=-1;
		}
	}
	purgefrags(self);
}
